<?php

/**
 *
 * @package    EasyAds
 * @author     Abude Bayassi <abude.bayassi@easyads.io>
 * @link       https://www.easyads.io
 * @copyright  2017 EasyAds (https://www.easyads.io)
 * @license    https://www.easyads.io
 * @since      1.0
 */

namespace app\controllers;

use app\fieldbuilder\Type;
use app\models\Listing;
use app\models\ListingFavorite;
use app\models\ListingImage;
use app\models\ListingPackage;
use app\models\ListingStat;
use app\models\Category;
use app\models\CategoryField;
use app\models\CategoryFieldValue;
use app\models\Customer;
use app\models\Gateway;
use app\models\Location;
use app\models\Order;
use app\models\OrderTax;
use app\models\OrderTransaction;
use app\models\Tax;
use app\models\Zone;
use yii\db\Expression;
use yii\helpers\ArrayHelper;
use yii\web\Response;
use yii\web\NotFoundHttpException;
use app\yii\web\Controller;
use app\helpers\CommonHelper;
use yii\helpers\Url;

/**
 * Class ListingController
 * @package app\controllers
 */
class ListingController extends Controller
{
    /**
     * init
     */
    public function init()
    {
        parent::init();
        app()->trigger('app.controller.ad.init');
    }

    /**
     * @param $slug
     * @return string
     * @throws NotFoundHttpException
     */
    public function actionIndex($slug)
    {
        $ad = $this->findAdBySlug($slug);

        if (!$ad->isActive && !$ad->isExpired) {
            if(!$ad->isOwnedBy(app()->customer->id)){
                throw new NotFoundHttpException(t('app', 'The ad is not active yet, please try again later.'));
            }
            notify()->addWarning(t('app', 'Your ad is not active yet and it\'s visible only to you!'));
        }

        if($ad->isExpired) {
            notify()->addWarning(t('app', 'This ad is expired!'));
        }

        app()->view->registerMetaTag([
            'name'    => 'description',
            'content' => Listing::getMetaDescription($ad)
        ]);

        ListingStat::track(ListingStat::TOTAL_VIEWS, $ad->listing_id);

        app()->view->title = $ad->title;

        // hook action for adding ad image to facebook meta object in head
        $mainImage = $ad->mainImage->image;
        app()->on('app.header.beforeScripts', function() use ($mainImage) {
            echo '<meta name="og:image" content="'.Url::base(true) . $mainImage.'"/>';
        });

        // #INFO
        // relation with getRelation gets the whole ActiveQuery, that you can manipulate
        // if you get by $ad->relation, you will have a result of ActiveRecord

        return $this->render('view', [
            'ad'                => $ad,
            'images'            => $ad->getImages()->orderBy(['sort_order'=>SORT_ASC])->all(),
            'categories'        => new Category(),
        ]);
    }

    /**
     * @param $slug
     * @return string|Response
     */
    public function actionPreview($slug)
    {
        if (app()->customer->isGuest == true) {
            return $this->redirect(['/']);
        }

        $ad = $this->findAdBySlug($slug);

        if (!$ad->isOwnedBy(app()->customer->id)) {
            return $this->redirect(['/']);
        }

        app()->view->title = t('app','Preview - ') . $ad->title;

        return $this->render('preview', [
            'ad'                => $ad,
            'images'            => $ad->getImages()->orderBy(['sort_order'=>SORT_ASC])->all(),
            'categories'        => new Category(),
        ]);
    }

    /**
     * @return string|Response
     */
    public function actionPost()
    {
        if (app()->customer->isGuest) {
            notify()->addWarning(t('app', 'Please login before performing this action.'));
            return $this->redirect(['/account/login']);
        }

        $ad = new Listing();
        $images = new ListingImage();
        $location = new Location();
        $categories = Category::find()->where(['status' => Category::STATUS_ACTIVE])->all();
        $customer = Customer::findOne(app()->customer->id);
        $customer->scenario = 'post_ad';

        if ($ad->load(request()->post()) &&
            $location->load(request()->post()) &&
            $customer->load(request()->post()) &&
            $images->load(request()->post())
        ) {
            $transaction = db()->beginTransaction();
            $error = false;
            try {
                $this->saveDraft($ad, $location, $customer, $images);
                $transaction->commit();
            } catch (\Exception $e) {
                notify()->addError($e->getMessage());
                $error = true;
                $transaction->rollBack();
            }
            if (!$error) {
                $target = (request()->post('post', 'no') == '') ? 'package' : 'preview';
                return $this->redirect(url(['/listing/' . $target, 'slug' => $ad->slug]));
            }
        }

        app()->view->title = t('app','Post Ad') . ' - ' . options()->get('app.settings.common.siteName', 'EasyAds');

        return $this->render('form', [
            'action'        => 'create',
            'ad'            => $ad,
            'images'        => $images,
            'location'      => $location,
            'categories'    => $categories,
            'customer'      => $customer,
        ]);
    }

    /**
     * @param $slug
     * @return string|Response
     */
    public function actionUpdate($slug)
    {
        if (app()->customer->isGuest) {
            return $this->redirect(['/listing/post']);
        }

        $ad = Listing::find()->where(['slug' => $slug])->one();
        if (empty($ad)) {
            return $this->redirect(['/listing/post']);
        }

        $customer = $ad->customer;
        $customer->scenario = 'post_ad';
        if (!$ad->isOwnedBy(app()->customer->id)) {
            return $this->redirect(['/listing/post']);
        }

        $uploadedImages  = $ad->images;
        $images          = new ListingImage();
        $location        = $ad->location;
        $categories      = Category::find()->where(['status' => Category::STATUS_ACTIVE])->all();
        $fields          = $ad->categoryFieldValues;

        if ($ad->load(request()->post()) &&
            $location->load(request()->post()) &&
            $customer->load(request()->post()) &&
            $images->load(request()->post())
        ) {
            $transaction = db()->beginTransaction();
            $error = false;
            try {
                $this->saveDraft($ad, $location, $customer, $images);
                $transaction->commit();

            } catch (\Exception $e) {
                notify()->addError($e->getMessage());
                $error = true;
                $transaction->rollBack();

            }
            if (!$error) {
                $target = (request()->post('post', 'no') == '') ? 'package' : 'preview';
                return $this->redirect(url(['/listing/' . $target, 'slug' => $ad->slug]));
            }
        }


        app()->view->title = t('app','Update Ad') . ' - ' . options()->get('app.settings.common.siteName', 'EasyAds');

        return $this->render('form', [
            'action'            => 'update',
            'ad'                => $ad,
            'images'            => $images,
            'location'          => $location,
            'fields'            => $fields,
            'categories'        => $categories,
            'customer'          => $customer,
            'uploadedImages'    => $uploadedImages,
        ]);
    }

    /**
     * @param $ad
     * @param $location
     * @param $customer
     * @param $images
     * @return mixed
     * @throws \Exception
     */
    protected function saveDraft($ad, $location, $customer, $images)
    {

        if (!($location->save())) {
            throw new \Exception(t('app', 'Your form has a few errors, please fix them and try again!'));
        }
        if (!($customer->save())) {
            throw new \Exception(t('app', 'Your form has a few errors, please fix them and try again!'));
        }
        $ad->location_id = $location->location_id;
        $ad->status = $ad::STATUS_DRAFT;

        if (!$ad->save()) {
            throw new \Exception(t('app', 'Your form has a few errors, please fix them and try again!'));
        }

        $oldFields = $ad->categoryFieldValues;
        if (!empty($oldFields)) {
            foreach ($oldFields as $oldField) {
                $oldField->delete();
            }
        }

        $fieldsPost = request()->post('CategoryField');
        if (!empty($fieldsPost)) {
            foreach ($fieldsPost as $fieldId=>$fieldPost) {
                $fields = new CategoryFieldValue();
                $fields->field_id = (int)$fieldId;
                $fields->listing_id = (int)$ad->listing_id;
                $fields->value = $fieldPost;
                $fields->save();
            }
        }
        $images->handleUploadedImages((int)$ad->listing_id);

        return $ad->listing_id;
    }

    /**
     * @param $slug
     * @return string|Response
     */
    public function actionPackage($slug)
    {
        if (app()->customer->isGuest) {
            return $this->redirect(['/listing/post']);
        }

        $ad = Listing::find()->where(['slug'=>$slug])->one();
        if (empty($ad)) {
            return $this->redirect(['/listing/post']);
        }

        $customer = $ad->customer;
        $customer->scenario = 'post_ad';
        if (!$ad->isOwnedBy(app()->customer->id)) {
            return $this->redirect(['/listing/post']);
        }

        $location = $ad->location;
        $packages = ListingPackage::find()->all();

        if (request()->post()) {
            $transaction = db()->beginTransaction();
            $error = false;
            try {
                $adPost = request()->post('Listing');
                $packageId = ArrayHelper::getValue($adPost, 'package_id');
                if (empty($packageId)) {
                    throw new \Exception(t('app', 'Please select a package first to finish your posting.'));
                }

                $package        = ListingPackage::find()->where(['package_id'=>$packageId])->one();
                $locationTax    = Tax::calculateTax($ad->location->country_id, $ad->location->zone_id, $package->price);
                $cart           = CommonHelper::getCartTotal($locationTax, $package->price);
                $cartTotal      = $cart['total'];

                //make new order
                $order = Order::createNew($ad, $package, $cart);
                if (empty($order)) {
                    throw new \Exception(t('app', 'Something went wrong with your order, please try again later.'));
                }
                //populate OrderTax
                if (!empty($locationTax)) {
                    foreach ($locationTax as $tax) {

                        $orderTax               = new OrderTax();
                        $orderTax->order_id     = $order->order_id;
                        $orderTax->tax_name     = $tax->name;
                        $orderTax->tax_percent  = $tax->percent;
                        $orderTax->tax_price    = $tax->price;
                        $orderTax->save(false);
                    }
                }
                // process payment methods and purchase action in the handlePayment
                if ($package->price > 0) {

                    //make new transaction
                    $orderTransaction = OrderTransaction::createNew($order);
                    if (empty($orderTransaction)) {
                        throw new \Exception(t('app', 'Something went wrong with your order transaction, please try again later.'));
                    }

                    if (request()->post('paymentGateway')) {
                        $event = new \app\yii\base\Event();
                        $event->params = [
                            'transaction'   => $orderTransaction,
                            'order'         => $order,
                            'customer'      => $customer,
                            'cartTotal'     => $cartTotal,
                            'location'      => $location
                        ];
                        app()->trigger('app.controller.ad.package.handlePayment', $event);
                    }
                } else {
                    // make paid status for free packages
                    $order->status = Order::STATUS_PAID;
                    $order->save(false);
                }
                // For methods with redirect
                if (!empty($event->params['redirect'])) {
                    $ad->package_id = $packageId;
                    $ad->save(false);
                    $transaction->commit();
                    return $this->redirect($event->params['redirect']);
                }

                //Continue processing if not a method with redirect
                //Check status of order after getting back from payment procedure
                if ($order->status != Order::STATUS_PAID) {
                    throw new \Exception(t('app', 'Something went wrong with your order verification, please try again later.'));
                }

                //update ad dates
                $ad->promo_expire_at = null;
                $ad->listing_expire_at = new Expression('NOW() + INTERVAL ' . (int)$package->listing_days . ' DAY');
                if ((int)$package->promo_days > 0) {
                    $ad->promo_expire_at = new Expression('NOW() + INTERVAL ' . (int)$package->promo_days . ' DAY');
                }

                $ad->remaining_auto_renewal = 0;
                if ((int)$package->auto_renewal > 0) {
                    $ad->remaining_auto_renewal = $package->auto_renewal;
                }

                //Check admin approval option to set the right status for ad
                $adminApproval = options()->get('app.settings.common.adminApprovalAds', 1);
                if ($adminApproval > 0) {
                    $ad->status = Listing::STATUS_PENDING_APPROVAL;
                    Listing::sendWaitingApprovalEmail($ad);
                } else {
                    Listing::sendAdActivateEmail($ad);
                    $ad->status = Listing::STATUS_ACTIVE;
                }

                $ad->package_id = $packageId;
                $ad->save(false);
                $transaction->commit();
            } catch (\Exception $e) {
                notify()->addError($e->getMessage());
                $error = true;
                $transaction->rollBack();
            }

            if (!$error) {
                notify()->addSuccess(t('app','Your ad was successfully created and will be active soon'));
                return $this->redirect('/account/my-listings');
            }
        }
        app()->view->title = t('app','Package') . ' - ' . $ad->title;

        return $this->render('form_package', [
            'ad'        => $ad,
            'packages'  => $packages,
        ]);
    }

    /**
     * @return array|Response
     */
    public function actionGetSummary()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $countryId      = request()->post('country_id');
        $zoneId         = request()->post('zone_id');
        $price          = request()->post('price');

        $locationTax = Tax::calculateTax($countryId, $zoneId, $price);

        $cart = CommonHelper::getCartTotal($locationTax, $price);

        return ['result' => 'success', 'html' => $this->renderPartial('summary',
                [
                    'subtotal' => $cart['subtotal'],
                    'tax'      => $cart['locationTax'],
                    'total'    => $cart['total'],
                ]
            )
        ];
    }

    /**
     * @return array|Response
     */
    public function actionGetCategoryFields()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $categoryId = request()->post('category_id');
        $adId       = request()->post('listing_id');
        $categoryFields = CategoryField::find()->where(['category_id'=>$categoryId])->orderBy(['sort_order'=>SORT_ASC])->all();

        $typeData = [];
        foreach ($categoryFields as $field) {
            $typeData[] = [
                'type' => $field->type,
                'field' => $field,
            ];
        }

        foreach ($typeData as $data) {
            $type  = $data['type'];
            $field = $data['field'];
            if (!is_file(\Yii::getAlias('@'. str_replace('\\', '/', $type->class_name) . '.php'))) {
                continue;
            }
            $className = $type->class_name;
            $component = new $className();

            if (!($component instanceof Type)) {
                continue;
            }

            $component->field = $field;
            $component->params = [
                'categoryId' => $categoryId,
                'adId' => $adId,
            ];
            $component->handleFrontendFormDisplay();
        }

        return ['result' => 'success', 'html' => $this->renderPartial('custom-fields')];
    }

    /**
     * @return array|Response
     */
    public function actionGetCountryZones()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $countryId = request()->post('country_id');
        if (empty($countryId)) {
            return ['result' => 'error', 'html' => t('app', 'Something went wrong...')];
        }

        $countryZones = Zone::find()->select(['zone_id','name'])->where(['country_id' => (int)$countryId])->all();

        return ['result' => 'success', 'response' => $countryZones];
    }

    /**
     * @return array|Response
     */
    public function actionRemoveAdImage()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $imageId = request()->post('key');
        if (empty($imageId)) {
            return ['result' => 'error', 'html' => t('app', 'Something went wrong...')];
        }

        $imageId = ListingImage::findOne(['image_id' => $imageId]);
        if (empty($imageId)) {
            return ['result' => 'error', 'html' => t('app', 'Something went wrong...')];
        }

        // if all images are deleted then the ad is not active anymore
        $countImages = ListingImage::find()->where(['listing_id' => $imageId->listing_id])->count();
        if ($countImages == 1) {
            $ad = Listing::find()->where(['listing_id' => $imageId->listing_id])->one();
            $ad->status = Listing::STATUS_DRAFT;
            $ad->save(false);
        }

        $imageId->delete();

        return ['result' => 'success', 'response' => $imageId];
    }

    /**
     * @return array|Response
     */
    public function actionSortAdImages()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $images = request()->post('images');
        $images = json_decode($images);

        if (empty($images)) {
            return ['result' => 'error', 'html' => t('app', 'Something went wrong...')];
        }

        foreach ($images as $sortOrder => $image) {
            $_image = ListingImage::findOne(['image_id' => $image->key]);
            $_image->sort_order = $sortOrder + 1;
            $_image->save(false);
        }

        return ['result' => 'success'];
    }

    /**
     * @return array|Response
     */
    public function actionGetCustomerContact()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $customerId = request()->post('customer_id');
        if (empty($customerId)) {
            return ['result' => 'error', 'html' => t('app', 'Something went wrong...')];
        }

        $customerEmail = Customer::find()->select(['email', 'phone'])->where(['customer_id' => $customerId])->one();

        return ['result' => 'success', 'response' => $customerEmail];
    }

    /**
     * @return array|Response
     */
    public function actionToggleFavorite()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        if (app()->customer->isGuest) {
            return ['result' => 'error', 'msg' => t('app', 'You need to be logged in to add favorites')];
        }

        $customer_id = app()->customer->id;
        if (empty($customer_id)) {
            return ['result' => 'error', 'msg' => t('app', 'Something went wrong...')];
        }

        $adId = request()->post('listing_id');
        $ad = Listing::findOne($adId);
        if (empty($ad)) {
            return ['result' => 'error', 'msg' => t('app', 'Something went wrong...')];
        }

        $adFavorite = ListingFavorite::find()->where(['listing_id' => $adId, 'customer_id' => $customer_id])->one();
        if (!empty($adFavorite)) {
            $adFavorite->delete();

            return ['result' => 'success', 'action' =>'added', 'msg' => t('app', 'Removed from Favorites')];
        }

        $adFavorite              = new ListingFavorite();
        $adFavorite->listing_id       = $adId;
        $adFavorite->customer_id = $customer_id;
        $adFavorite->save();

        return ['result' => 'success', 'action' =>'removed', 'msg' => t('app', 'Added to Favorites')];
    }

    /**
     * @return array|Response
     */
    public function actionDelete()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        if (app()->customer->isGuest) {
            return ['result' => 'error', 'msg' => t('app', 'You need to be logged in to make this action')];
        }

        $customer_id = app()->customer->id;
        if (empty($customer_id)) {
            return ['result' => 'error', 'msg' => t('app', 'Something went wrong...')];
        }

        $adId = request()->post('listing_id');
        $ad = Listing::find()->where(['listing_id'=>$adId, 'customer_id' => $customer_id])->one();
        if (empty($ad)) {
            return ['result' => 'error', 'msg' => t('app', 'Something went wrong...')];
        }

        if ($ad->delete()) {
            return ['result' => 'success', 'msg' => t('app', 'Your Ad was deleted!')];
        }
        return ['result' => 'error', 'msg' => t('app', 'We couldn\'t delete your ad, please try again later')];
    }

    /**
     * @return array|Response
     */
    public function actionTrackStats()
    {
        /* allow only ajax calls */
        if (!request()->isAjax) {
            return $this->redirect(['listing/post']);
        }

        /* set the output to json */
        response()->format = Response::FORMAT_JSON;

        $adId = request()->post('listing_id');
        $statsType = request()->post('stats_type');
        $ad = Listing::find()->where(['listing_id'=>$adId])->one();
        if (empty($ad)) {
            return ['result' => 'error', 'msg' => t('app', 'Something went wrong...')];
        }

        ListingStat::track($statsType, $adId);
        return ['result' => 'success'];
    }

    /**
     * @param $slug
     * @return static
     * @throws NotFoundHttpException
     */
    protected function findAdBySlug($slug)
    {
        if (($ad = Listing::findOne(['slug' => $slug])) !== null) {
            return $ad;
        }

        throw new NotFoundHttpException(t('app', 'The requested page does not exist.'));
    }

    /**
     * @param $id
     * @return static
     * @throws NotFoundHttpException
     */
    protected function findModel($id)
    {
        if (($model = Listing::findOne($id)) !== null) {
            return $model;
        }
        throw new NotFoundHttpException(t('app', 'The requested page does not exist.'));
    }
}