<?php

/**
 *
 * @package    EasyAds
 * @author     Abude Bayassi <abude.bayassi@easyads.io>
 * @link       https://www.easyads.io
 * @copyright  2017 EasyAds (https://www.easyads.io)
 * @license    https://www.easyads.io
 * @since      1.0
 */

namespace app\controllers;

use app\models\ListingSearch;
use app\models\Category;
use app\models\Zone;
use yii\db\Expression;
use app\yii\web\Controller;
use yii\filters\VerbFilter;
use yii\web\NotFoundHttpException;

/**
 * Class SiteController
 * @package app\controllers
 */
class SiteController extends Controller
{
    const ADS_PER_PAGE = 20;

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class'   => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * @inheritdoc
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */
    public function actionIndex()
    {
        $searchModel = new ListingSearch();
        $topCatsWithFirstLevelChildren = Category::getTopCatsWithFirstLevelChildren();

        // Meta tags
        app()->view->registerMetaTag([
            'name'    => 'keywords',
            'content' => options()->get('app.settings.common.siteKeywords', 'EasyAds')
        ]);
        app()->view->registerMetaTag([
            'name'    => 'description',
            'content' => options()->get('app.settings.common.siteDescription', 'EasyAds')
        ]);

        app()->view->title = options()->get('app.settings.common.siteName', 'EasyAds');

        return $this->render('index', [
            'searchModel' => $searchModel,
            'categories'  => $topCatsWithFirstLevelChildren
        ]);
    }

    /**
     * Displays search page.
     *
     * @return string
     */
    public function actionSearch()
    {
        $searchModel = new ListingSearch();
        $categories = Category::find()->where(['status' => Category::STATUS_ACTIVE])->all();

        $categoryPlaceholderText = t('app', 'Choose Category');
        if ($slug = request()->get('slug')) {
            $chosenCategory = Category::findCategoryBySlug($slug);
            if ($chosenCategory) {
                $categoryPlaceholderText = Icon::make($chosenCategory->icon) . ' ' .html_encode($chosenCategory->name);
            }
        }

        // ads
        $AdsProvider = $searchModel->categorySearch(request()->queryParams);
        $AdsProvider->query->andWhere(['category_id' => $categories])
            ->orderBy(['promo_expire_at' => SORT_DESC, 'created_at' => SORT_DESC]);
        $AdsProvider->sort = ['defaultOrder' => ['promo_expire_at' => SORT_DESC]];
        $AdsProvider->pagination = [
            'defaultPageSize' => self::ADS_PER_PAGE,
        ];

        // select location details if filter is not empty
        $locationDetails = '';
        if (isset(request()->queryParams['ListingSearch']) && !empty(request()->queryParams['ListingSearch']['zoneId'])) {
            $zone = Zone::find()->with('country')->where(['zone_id' => request()->queryParams['ListingSearch']['zoneId']])->one();
            if ($zone) {
                $locationDetails = $zone->name . ', ' . $zone->country->name;
            }
        }

        app()->view->title = t('app', 'Search') . ' - ' . options()->get('app.settings.common.siteName', 'EasyAds');

        return $this->render('search-results', [
            'searchModel'               => $searchModel,
            'categories'                => $categories,
            'categoryPlaceholderText'   => $categoryPlaceholderText,
            'adsProvider'               => $AdsProvider,
            'locationDetails'           => $locationDetails,
            'isNothingFound'            => !$AdsProvider->getCount(),
        ]);
    }

    /**
     * @return string
     */
    public function actionOffline()
    {
        if(options()->get('app.settings.common.siteStatus', 1) == 1) {
            $this->redirect(['/']);
        }

        app()->view->title = t('app', 'Offline') . ' - ' . options()->get('app.settings.common.siteName', 'EasyAds');

        return $this->render('offline', [
           'message' =>  options()->get('app.settings.common.siteOfflineMessage', t('app', 'Application is offline, please try again later!'))
        ]);
    }

    /**
     * @param $url
     * @return \yii\web\Response
     * @throws NotFoundHttpException
     */
    public function actionRedirect($url)
    {
        if (substr($url, -1) == '/') {
            return $this->redirect(app()->getRequest()->getBaseUrl() . '/' . rtrim($url, '/'), 301);
        }
        throw new NotFoundHttpException(t('app', 'The requested page does not exist.'));
    }

}