<?php

/**
 *
 * @package    EasyAds
 * @author     Abude Bayassi <abude.bayassi@easyads.io>
 * @link       https://www.easyads.io
 * @copyright  2017 EasyAds (https://www.easyads.io)
 * @license    https://www.easyads.io
 * @since      1.0
 */

namespace app\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use yii\db\Expression;

/**
 * Class ListingSearch
 * @package app\common\models
 */
class ListingSearch extends Listing
{
    public $searchPhrase;
    public $zoneId;
    public $minPrice;
    public $maxPrice;
    public $store;
    public $categorySlug;

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['title', 'package_id', 'customer_id', 'location_id', 'category_id', 'currency_id', 'status', 'searchPhrase', 'zoneId', 'categorySlug', 'store'], 'safe'],
            [['price', 'remaining_auto_renewal', 'minPrice', 'maxPrice', 'negotiable'], 'integer'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function scenarios()
    {
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        $query = Listing::find()->alias('t');
        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort'       => ['defaultOrder' => ['updated_at' => SORT_DESC]],
        ]);
        $this->load($params);
        if (!$this->validate()) {
            return $dataProvider;
        }

        if (strlen($this->customer_id) >= 1) {
            $query->joinWith(['customer' => function ($query) {
                $query->from(Customer::tableName() . ' c1');
            }], true, 'INNER JOIN')
                ->andWhere(['like', 'c1.first_name', $this->customer_id])
                ->orWhere(['like', 'c1.last_name', $this->customer_id]);
        }

        if (strlen($this->store) >= 1) {
            $query->joinWith(['store' => function ($query) {
                $query->from(CustomerStore::tableName() . ' c6');
            }], true, 'INNER JOIN')->andWhere(['like', 'c6.store_name', $this->store])->andWhere(['=', 'c6.status', CustomerStore::STATUS_ACTIVE]);
        }

        if (strlen($this->category_id) >= 1) {
            $query->joinWith(['category' => function ($query) {
                $query->from(Category::tableName() . ' c2');
            }], true, 'INNER JOIN')->andWhere(['like', 'c2.name', $this->category_id]);
        }

        if (strlen($this->currency_id) >= 1) {
            $query->joinWith(['currency' => function ($query) {
                $query->from(Currency::tableName() . ' c3');
            }], true, 'INNER JOIN')->andWhere(['like', 'c3.name', $this->currency_id]);
        }

        if (strlen($this->location_id) >= 1) {
            $query->joinWith(['location' => function ($query) {
                $query->from(Location::tableName() . ' c4');
            }], true, 'INNER JOIN')->andWhere(['like', 'c4.city', $this->location_id]);
        }

        if (strlen($this->package_id) >= 1) {
            $query->joinWith(['package' => function ($query) {
                $query->from(ListingPackage::tableName() . ' c5');
            }], true, 'INNER JOIN')->andWhere(['like', 'c5.title', $this->package_id]);
        }

        // grid filtering conditions
        $query->andFilterWhere(['like', 't.title', $this->title])
            ->andFilterWhere(['=', 't.price', $this->price])
            ->andFilterWhere(['=', 't.remaining_auto_renewal', $this->remaining_auto_renewal])
            ->andFilterWhere(['like', 't.created_at', $this->created_at]);

        return $dataProvider;
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function categorySearch($params)
    {
        $query = Listing::find()->alias('t')
            ->with(['mainImage', 'category', 'favorite'])
            ->where(['>', 't.listing_expire_at', new Expression('NOW()')])
            ->andWhere(['t.status' => 'active']);

        $query->joinWith(['currency', 'location.zone', 'location.country']);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
        ]);

        $this->load($params);
        if (!$this->validate()) {
            return $dataProvider;
        }

        // custom fields filters
        if (isset($params['CategoryField'])) {
            $customFieldsFilters = $params['CategoryField'];
            $filterFields = [];
            foreach ($customFieldsFilters as $fieldId => $filters) {
                if (is_array($filters)) {
                    foreach ($filters as $filter) {
                        if (!empty($filter)) {
                            $filterFields[$fieldId] = $filters;
                        }
                    }
                } else {
                    if (!empty($filters)) {
                        $filterFields[$fieldId] = $filters;
                    }
                }
            }

            if (!empty($filterFields)) {
                foreach ($filterFields as $fieldId => $filters) {
                    if (is_array($filters)) {
                        $alias = 'cf' . $fieldId;
                        $query->joinWith(['categoryFieldValues ' . $alias], true, 'INNER JOIN')
                            ->andWhere([$alias . '.field_id' => $fieldId]);

                        if (!empty($filters['min'])) {
                            $query->andWhere(['>=', $alias . '.value', $filters['min']]);
                        }
                        if (!empty($filters['max'])) {
                            $query->andWhere(['<=', $alias . '.value', $filters['max']]);
                        }
                    } else {
                        $alias = 'cf' . $fieldId;
                        $query->joinWith(['categoryFieldValues ' . $alias], true, 'INNER JOIN')
                            ->andWhere([$alias . '.field_id' => $fieldId])
                            ->andWhere([$alias . '.value' => $filters]);
                    }
                }
            }
        }

        if ($this->zoneId) {
            $query->andFilterWhere(['=', '{{%zone}}.zone_id', $this->zoneId]);
        }

        if (trim($this->searchPhrase) != '') {

            $query->andWhere('(t.title LIKE :searchPhrase OR t.description LIKE :searchPhrase)', [
                ':searchPhrase' => '%'.$this->searchPhrase.'%'
            ]);
        }

        // grid filtering conditions
        $query->andFilterWhere(['=', 't.remaining_auto_renewal', $this->remaining_auto_renewal])
            ->andFilterWhere(['=', 't.negotiable', $this->negotiable])
            ->andFilterWhere(['=', '{{%currency}}.currency_id', $this->currency_id])
            ->andFilterWhere(['>=', 't.price', $this->minPrice])
            ->andFilterWhere(['<=', 't.price', $this->maxPrice]);

        return $dataProvider;
    }
}
